<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoOcupacao extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrorg              = Utils::getParamters()->getNrorg();
        $nrvinculom         = $this->getSolicitacao()->getNrvinculom();
        $dsobssolicitacao   = $this->getSolicitacao()->getDsobssolicitacao();
        $dtcompetencia      = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO,true);
        
        // OCUPAÇÃO
        $nrocupacaom_ocu    = $itens['NROCUPACAOM']     ?? $itens['NROCUPACAOM_OCU'] ?? null; 
        $nrmotivoocupa      = $itens['NRMOTIVOOCUPA']   ?? null; 
        $nrtpmodalidsal     = $itens['NRTPMODALIDSAL']  ?? null; 
        $nrtipoocupacao     = $itens['NRTIPOOCUPACAO']  ?? null;
        $dtiniocupacao      = isset($itens['DTINIOCUPACAO']) ? DateUtil::getDataDeString(substr($itens['DTINIOCUPACAO'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;

        // SALÁRIO
        $dtaltesalario  = isset($itens['DTALTESALARIO']) ? DateUtil::getDataDeString(substr($itens['DTALTESALARIO'],0,10), 'd/m/Y', true) : null;
        $nrtipoaltera   = $itens['NRTIPOALTERA']        ?? null;
        $idtpsalario    = $itens['IDTPSALARIO']         ?? null;
        $nrocorrencia   = $itens['NROCORRENCIA']        ?? null;
        $novoVlrSalario = $itens['VRSALARIO']           ?? $itens['NOVOVALORSALARIO'] ?? null;
        
        $result = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTEOCUPACAO)->findBy(array("nrorg"=> Utils::getParamters()->getNrorg(),"nrvinculom" => $nrvinculom,"dtiniocupacao" => $dtiniocupacao));

        if($result)
            throw new \Exception("Já existe uma alteração de ocupação para este vinculo nesta data");
        $this->atualizaAlteOcupa($nrvinculom,DateUtil::subtraiIntervalo($dtiniocupacao,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),Utils::getParamters()->getCdoperador());
        
        $gpeAlteocupacao = Repositories::GPE_ALTEOCUPACAO;
        $alteOcupacao = new $gpeAlteocupacao;
        $alteOcupacao->setNrorg(Utils::getParamters()->getNrorg());
        $alteOcupacao->setNrvinculom($nrvinculom);
        $alteOcupacao->setDsobservacao($dsobssolicitacao);
        $alteOcupacao->setDtiniocupacao($dtiniocupacao);
        $alteOcupacao->setNrmotivoocupa($nrmotivoocupa);
        $alteOcupacao->setNrocupacaom($nrocupacaom_ocu);
        $alteOcupacao->setNrtipoocupacao($nrtipoocupacao);
        $alteOcupacao->setNrtpmodalidsal($nrtpmodalidsal);
        $alteOcupacao->setDtinclusao(DateUtil::getDataAtual(false));
        $alteOcupacao->setNrorginclusao(Utils::getParamters()->getNrorg());
        $alteOcupacao->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $alteOcupacao->setNralteocupacao(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_ALTEOCUPACAO', 1, 12, true));
        Utils::getEntityManager()->persist($alteOcupacao);
        Utils::getEntityManager()->flush();
        
        $vinc = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, Utils::getParamters()->getDtmescomptencDiaCorrente());
        if($vinc->getDtmescompetenc() == Utils::getParamters()->getDtmescomptencDiaCorrente()){
            $vinc->setDtultatu(DateUtil::getDataAtual());
            $vinc->setNrorgultatu(Utils::getParamters()->getNrorg());
            $vinc->setCdoperultatu(Utils::getParamters()->getCdoperador());
            $vinc->setNrcargo($nrocupacaom_ocu);
            Utils::getEntityManager()->persist($vinc);
            Utils::getEntityManager()->flush();
        }
        else
        ProcessamentoSolicitacaoUtil::novoHistoricoVinculo($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $nrocupacaom_ocu, $vinc->getNrescalatrabm(),
            $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
            $vinc->getIdtppagamento(), $vinc->getNrestrutlegal(), $vinc->getNrestrutgeren(), $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
            $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
            $vinc->getNrestrutsind(), $vinc->getConfidencial()
        );


        if($nrtipoaltera && $novoVlrSalario && $idtpsalario) {
            
            if (!$dtaltesalario) {
                throw new \Exception("Solicitação sem data ou com data de alteração inválida. Favor reprovar e incluir uma nova solicitação.");
            }
            
            if (!$nrtipoaltera) {
                throw new \Exception("Solicitação sem tipo de alteração informado. Favor reprovar e incluir uma nova solicitação.");
            }
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeAltesalario $alteSalario */
            $gpeAltesalrio = Repositories::GPE_ALTESALARIO;
            $alteSalario = new $gpeAltesalrio;
            $alteSalario->setNrorg(Utils::getParamters()->getNrorg());
            $alteSalario->setNrvinculom($nrvinculom);
            $alteSalario->setDtaltesalario($dtaltesalario);
            $alteSalario->setNrocorrencia($nrocorrencia);
            $alteSalario->setNrtipoaltera($nrtipoaltera);
            $alteSalario->setNrtpmodalidsal($nrtpmodalidsal);
            $alteSalario->setIdtpsalario($idtpsalario);
            $alteSalario->setVrsalario($novoVlrSalario);
            $alteSalario->setDtinclusao(DateUtil::getDataAtual(false));
            $alteSalario->setNrorginclusao(Utils::getParamters()->getNrorg());
            $alteSalario->setCdoperinclusao(Utils::getParamters()->getCdoperador());
            $alteSalario->setNraltesalario(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_ALTESALARIO', 1, 12, true));
            Utils::getEntityManager()->persist($alteSalario);
            Utils::getEntityManager()->flush();
        }

    }
    
    public function rejeitar() {}
    
    public function atualizaAlteOcupa($nrvinculom, $dtfimocupacao,$dtatual,$cdoperador) {
        $sql = <<<SQL
            UPDATE GPE_ALTEOCUPACAO
            SET DTFIMOCUPACAO = :P_DTFIMOCUPACAO,
            DTULTATU = :P_DTATUAL,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
            WHERE DTINIOCUPACAO = (
                SELECT MAX(DTINIOCUPACAO) 
                FROM GPE_ALTEOCUPACAO 
                WHERE NRORG = :P_NRORG 
                AND NRVINCULOM = :P_NRVINCULOM 
                AND DTFIMOCUPACAO IS NULL
                )
            AND NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM 
            AND DTINIOCUPACAO >= :P_DTFIMOCUPACAO
            AND DTFIMOCUPACAO IS NULL
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"             => Utils::getParamters()->getNrorg(),
                "P_DTFIMOCUPACAO"     => $dtfimocupacao,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTATUAL"           => $dtatual,
                "P_CDOPERADOR"        => $cdoperador
            )
        );
    }
}