<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class AlteOcupacaoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }
    
    public function getAlteocupacao(
        array $params
    ) {
        $sql = <<<SQL
        SELECT
            GPE_ALTEOCUPACAO.NRALTEOCUPACAO,
            GPE_ALTEOCUPACAO.NROCUPACAOM,
            TO_CHAR(GPE_ALTEOCUPACAO.DTINIOCUPACAO, 'DD/MM/YYYY') AS DTINIOCUPACAO,
            TO_CHAR(GPE_ALTEOCUPACAO.DTFIMOCUPACAO, 'DD/MM/YYYY') AS DTFIMOCUPACAO,
            GPE_ALTEOCUPACAO.DTINCLUSAO,
            GPE_ALTEOCUPACAO.CDOPERINCLUSAO,
            GPE_ALTEOCUPACAO.NRORGINCLUSAO,
            GPE_ALTEOCUPACAO.DTULTATU,
            GPE_ALTEOCUPACAO.CDOPERULTATU,
            GPE_ALTEOCUPACAO.NRORGULTATU,
            VM.NMVINCULOM,
            VM.NRTIPOVINCULOM,
            OH.NMOCUPACAOH,
            GPE_CBO.NRCBO,
            GPE_CBO.NMCBO,
            VH.NRESTRUTGEREN,
            VH.NRESTRUTLEGAL,
            GPE_TIPOOCUPACAO.NMTIPOOCUPACAO,
            GPE_MOTIVOOCUPA.NMMOTIVOOCUPA,
            FPA_TPMODALIDSAL.NMTPMODALIDSAL,
            EML.NMESTRUTURAM   AS NMESTRUTLEGAL,
            EMG.NMESTRUTURAM   AS NMESTRUTGEREN,
            FX.DSFAIXA
        FROM
            GPE_ALTEOCUPACAO
            JOIN GPE_VINCULOM    VM ON VM.NRVINCULOM = GPE_ALTEOCUPACAO.NRVINCULOM
                                    AND VM.NRORG = GPE_ALTEOCUPACAO.NRORG
                                    AND (  VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC
                                          OR VM.DTRESCISAOVINC IS NULL
                                          OR :P_RESCISAO != 'A'
                                          OR :P_RESCISAO IS NULL )
                                    AND ( VM.DTRESCISAOVINC < :P_DTMESCOMPETENC
                                          OR :P_RESCISAO != 'R'
                                          OR :P_RESCISAO IS NULL )
            JOIN GPE_VINCULOH    VH ON VH.NRVINCULOM = VM.NRVINCULOM
                                    AND VH.NRORG = VM.NRORG
                                    AND VH.DTMESCOMPETENC = (
                SELECT
                    MAX(H1.DTMESCOMPETENC)
                FROM
                    GPE_VINCULOH H1
                WHERE
                    H1.NRORG = VH.NRORG
                    AND H1.NRVINCULOM = VH.NRVINCULOM
                    AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
            )
            JOIN GPE_OCUPACAOM   OM ON OM.NROCUPACAOM = GPE_ALTEOCUPACAO.NROCUPACAOM
                                     AND OM.NRORG = GPE_ALTEOCUPACAO.NRORG
            JOIN GPE_OCUPACAOH   OH ON OH.NROCUPACAOM = OM.NROCUPACAOM
                                     AND OH.NRORG = OM.NRORG
                                     AND OH.DTMESCOMPETENC = (
                SELECT
                    MAX(OH1.DTMESCOMPETENC)
                FROM
                    GPE_OCUPACAOH OH1
                WHERE
                    OH1.NROCUPACAOM = OH.NROCUPACAOM
                    AND OH1.NRORG = OH.NRORG
                    AND OH1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
            )
            LEFT JOIN GPE_CBO ON OH.NRCBO = GPE_CBO.NRCBO
            JOIN GPE_TIPOOCUPACAO ON GPE_TIPOOCUPACAO.NRTIPOOCUPACAO = GPE_ALTEOCUPACAO.NRTIPOOCUPACAO
                                     AND ( GPE_TIPOOCUPACAO.NRORG = GPE_ALTEOCUPACAO.NRORG
                                           OR GPE_TIPOOCUPACAO.NRORG = :P_NRORG_PADRAO )
            LEFT JOIN ESTRUTURAM      EMG ON EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
                                        AND EMG.NRORG = VH.NRORG
            LEFT JOIN ESTRUTURAM      EML ON EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                        AND EML.NRORG = VH.NRORG
            LEFT JOIN GPE_MOTIVOOCUPA ON GPE_MOTIVOOCUPA.NRMOTIVOOCUPA = GPE_ALTEOCUPACAO.NRMOTIVOOCUPA
                                         AND ( GPE_MOTIVOOCUPA.NRORG = GPE_ALTEOCUPACAO.NRORG
                                               OR GPE_MOTIVOOCUPA.NRORG = :P_NRORG_PADRAO )
            LEFT JOIN FPA_TPMODALIDSAL ON FPA_TPMODALIDSAL.NRTPMODALIDSAL = GPE_ALTEOCUPACAO.NRTPMODALIDSAL
                                          AND ( FPA_TPMODALIDSAL.NRORG = GPE_ALTEOCUPACAO.NRORG
                                                OR FPA_TPMODALIDSAL.NRORG = :P_NRORG_PADRAO )
            LEFT JOIN GSO_FAIXA       FX ON FX.NRFAIXA = GPE_ALTEOCUPACAO.NRFAIXA
                                      AND ( FX.NRORG = GPE_ALTEOCUPACAO.NRORG
                                            OR FX.NRORG = :P_NRORG_PADRAO )
        WHERE
            GPE_ALTEOCUPACAO.NRORG = :P_NRORG
            AND (GPE_ALTEOCUPACAO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
        ORDER BY
            VM.NMVINCULOM ASC,
            GPE_ALTEOCUPACAO.DTINIOCUPACAO DESC
SQL;

        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute($params);

        return $query->fetchAll();
    }
    
    public function getAlteracaoCargo(
        string $dtInicio, 
        string $dtFinal, 
        int $nrCodIntEstrutLegal,
        int $nrOrg
        
    ): array {
        $sql = <<<SQL
            SELECT
                VM.*,
                PH.*,
                VH.NRVINCULOH,
                VH.NRSITUFUNCM,
                VH.NRCARGO,
                VH.DTMESCOMPETENC,
                VH.NRESTRUTLEGAL,
                VH.NRESTRUTGEREN,
                CARGO.DTINIOCUPACAO AS DATALT,
                ESTRUTLEGAL.NMESTRUTURAM AS NMESTRUTLEGAL,
                ESTRUTLEGAL.CDINTESTRUTURA AS NUMEMP,
                ESTRUTGEREN.NMESTRUTURAM AS NMESTRUTGEREN,
                OH.CDINTEGRACAO AS CODCAR
            FROM
                GPE_PESSOA PE
                JOIN
                GPE_PESSOAH PH ON
                    PH.NRPESSOA = PE.NRPESSOA AND
                    PH.NRORG =  PE.NRORG
                LEFT JOIN
                GPE_VINCULOM VM ON
                    PE.NRPESSOA = VM.NRPESSOA AND
                    PE.NRORG = VM.NRORG
                LEFT JOIN
                GPE_VINCULOH VH ON
                    VH.NRVINCULOM = VM.NRVINCULOM AND
                    VH.NRORG = VM.NRORG AND
                    VH.DTMESCOMPETENC = (
                        SELECT 
                            MAX(VHH.DTMESCOMPETENC)
                        FROM 
                            GPE_VINCULOH VHH
                        WHERE 
                            VHH.NRVINCULOM = VH.NRVINCULOM AND
                            VHH.NRORG = VH.NRORG AND
                            VHH.DTMESCOMPETENC <= :P_DTFINAL
                    )
                LEFT JOIN
                GPE_OCUPACAOH OH ON
                    OH.NROCUPACAOM = VH.NRCARGO AND
                    OH.NRORG = VH.NRORG AND
                    OH.DTMESCOMPETENC = (
                        SELECT
                            MAX(OHH.DTMESCOMPETENC)
                        FROM
                            GPE_OCUPACAOH OHH
                        WHERE
                            OHH.NRORG = OH.NRORG AND
                            OHH.NROCUPACAOM = OH.NROCUPACAOM AND
                            OHH.DTMESCOMPETENC <=:P_DTFINAL
                    )
                LEFT JOIN 
                GPE_ALTEOCUPACAO CARGO ON
                    CARGO.NRORG = VH.NRORG AND
                    CARGO.NRVINCULOM = VH.NRVINCULOM
                LEFT JOIN
                ESTRUTURAM ESTRUTLEGAL ON
                    ESTRUTLEGAL.NRESTRUTURAM = VH.NRESTRUTLEGAL AND
                    ESTRUTLEGAL.NRORG = VH.NRORG
                LEFT JOIN
                ESTRUTURAM ESTRUTGEREN ON
                    ESTRUTGEREN.NRESTRUTURAM = VH.NRESTRUTGEREN AND
                    ESTRUTGEREN.NRORG = VH.NRORG
            WHERE
                PE.NRORG =:P_NRORG AND
                (
                    CARGO.DTINIOCUPACAO BETWEEN :P_DTINICIO AND :P_DTFINAL OR
                    CARGO.DTFIMOCUPACAO BETWEEN :P_DTINICIO AND :P_DTFINAL
                ) AND
                (
                    ESTRUTLEGAL.CDINTESTRUTURA = :P_NRCODINTESTRUTLEGAL
                ) AND
                (
                    (
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(PHH.DTMESCOMPETENC)
                            FROM GPE_PESSOAH PHH
                            WHERE
                                PHH.NRORG = PH.NRORG AND
                                PHH.NRPESSOA = PH.NRPESSOA AND
                                PHH.DTMESCOMPETENC <=:P_DTFINAL
                        ) AND
                        VM.NRVINCULOM IS NOT NULL
                    )
                )
            ORDER BY PH.NMPESSOA
SQL;

        $params = [
            'P_DTINICIO' => $dtInicio,
            'P_DTFINAL' => $dtFinal,
            'P_NRCODINTESTRUTLEGAL' => $nrCodIntEstrutLegal,
            'P_NRORG' => $nrOrg
        ];

        $std =$this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
}