<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;

use Ems\Repository\AlteOcupacaoRepository;

class AlteOcupacaoService {
    /** @var Environment */
    protected $environment;

    /** @var AlteOcupacaoRepository */
    protected $alteOcupacaoRepository;

    public function __construct(
        AlteOcupacaoRepository $alteOcupacaoRepository,
        Environment $environment
    ) {
        $this->alteOcupacaoRepository       = $alteOcupacaoRepository;
        $this->environment                  = $environment;
    }
    
    public function getAlteocupacao(
        string  $nrVinculo,
        string  $dtMesCompetenc = null,
        string  $todosVinculos = 'T'
    ) {
        $nrOrg = $this->environment->getNrOrg();
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();
        
        $dtMesCompetenc = empty($dtMesCompetenc) ? $this->environment->getDtmescomptencDiaCorrente() : $dtMesCompetenc;
        
        $params = array(
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_RESCISAO'        => $todosVinculos,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc,
            'P_NRVINCULOM'      => $nrVinculo,
        );
        
        return $this->alteOcupacaoRepository->getAlteocupacao($params);
    }
    
    public function getAlteracaoCargo(
        string $dtInicio, 
        string $dtFinal, 
        int $nrCodIntEstrutLegal
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $vinculos = $this->alteOcupacaoRepository->getAlteracaoCargo(
            $dtInicio, $dtFinal, $nrCodIntEstrutLegal, $nrOrg
        );
        
        $retorno;
        
        if (empty($vinculos))
                throw new \Exception("Não foi encontrado dados com os valores utilizados.");
        else if(!empty($vinculos)) {
            foreach($vinculos as $vinculo) {
                $retorno[] = array(
                    "eDatAlt" => $dtInicio,
                    "eNumEmp" => $vinculo['NRESTRUTLEGAL'],
                    "NumEmp" => $vinculo['NUMEMP'],
                    "NumCad" => $vinculo['CDMATRICULA'],
                    "NumCpf" => $vinculo['NRCPFPESSOA'],
                    "CodCar" => $vinculo['CODCAR'],
                    "DatAlt" => $vinculo['DATALT']
                );
            }
            
        } 
        
        
        return $retorno;
    }
}
